import unittest

from nose_parameterized import parameterized

import snap_modules


class SnapModulesTestCase(unittest.TestCase):
    def test_get_cores(self):
        self.assertEquals(("2.4.19", "2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1"), snap_modules.get_cores())

    def test_get_parts(self):
        expected = [
            "RF100",
            "RF200P81",
            "RF200PD1",
            "RF200PF1",
            "RF200PU1",
            "RF220SU",
            "RF220SU-EU",
            "RF220UF1",
            "RF266PC1",
            "RF266PU1",
            "SM200P81",
            "SM200PU1",
            "SM220UF1",
            "SM220UF1-EU",
            "SN220",
            "SS200"
        ]
        self.assertEquals(expected, sorted(snap_modules.get_parts()))

    @parameterized.expand([
        ("RF100", ("2.4.19", "2.5.6")),
        ("RF200PF1", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("RF200PD1", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("SM200PU1", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("SM200P81", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("RF200PU1", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("RF200P81", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("RF266PC1", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("RF266PU1", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("SS200", ("2.4.34", "2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("RF220SU", ("2.7.1", "2.7.2", "2.8.1")),
        ("RF220SU-EU", ("2.8.1",)),
        ("SM220UF1", ("2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("SM220UF1-EU", ("2.8.1",)),
        ("RF220UF1", ("2.5.6", "2.6.2", "2.6.9", "2.7.1", "2.7.2", "2.8.1")),
        ("SN220", ("2.7.1", "2.7.2", "2.8.1"))
    ])
    def test_get_supported_cores_for_part(self, part, supported_cores):
        self.assertEquals(supported_cores, snap_modules.get_supported_cores_for_part(part))

    def test_get_supported_cores_for_unknown_part(self):
        self.assertRaises(snap_modules.UnknownPartError, snap_modules.get_supported_cores_for_part, "RF266PFC1")

    @parameterized.expand([
        ("2.4.19", ["RF100"]),
        ("2.4.34", ["RF200P81", "RF200PD1", "RF200PF1", "RF200PU1", "RF266PC1", "RF266PU1",
                    "SM200P81", "SM200PU1", "SS200"]),
        ("2.5.6", ["RF100", "RF200P81", "RF200PD1", "RF200PF1", "RF200PU1", "RF220UF1", "RF266PC1", "RF266PU1",
                   "SM200P81", "SM200PU1", "SM220UF1", "SS200"]),
        ("2.6.2", ["RF200P81", "RF200PD1", "RF200PF1", "RF200PU1", "RF220UF1", "RF266PC1", "RF266PU1",
                   "SM200P81", "SM200PU1", "SM220UF1", "SS200"]),
        ("2.6.9", ["RF200P81", "RF200PD1", "RF200PF1", "RF200PU1", "RF220UF1", "RF266PC1", "RF266PU1",
                   "SM200P81", "SM200PU1", "SM220UF1", "SS200"]),
        ("2.7.1", ["RF200P81", "RF200PD1", "RF200PF1", "RF200PU1", "RF220SU", "RF220UF1", "RF266PC1", "RF266PU1",
                   "SM200P81", "SM200PU1", "SM220UF1", "SN220", "SS200"]),
        ("2.7.2", ["RF200P81", "RF200PD1", "RF200PF1", "RF200PU1", "RF220SU", "RF220UF1", "RF266PC1", "RF266PU1",
                   "SM200P81", "SM200PU1", "SM220UF1", "SN220", "SS200"]),
        ("2.8.1", ["RF200P81", "RF200PD1", "RF200PF1", "RF200PU1", "RF220SU", "RF220SU-EU", "RF220UF1",
                   "RF266PC1", "RF266PU1", "SM200P81", "SM200PU1", "SM220UF1", "SM220UF1-EU", "SN220", "SS200"]),
    ])
    def test_get_supported_parts_for_core(self, core, supported_parts):
        self.assertEquals(supported_parts, sorted(snap_modules.get_supported_parts_for_core(core)))

    def test_get_supported_parts_for_unknown_core(self):
        self.assertRaises(snap_modules.UnknownCoreError, snap_modules.get_supported_parts_for_core, "2.4.42")

    @parameterized.expand([
        ("RF100", "2.4.19", True),
        ("RF100", "2.4.34", False),
        ("RF100", "2.5.6", True),
    ])
    def test_is_supported_combination(self, part, core, supported):
        self.assertEquals(supported, snap_modules.is_supported_combination(part=part, core=core))

    def test_is_supported_combination_for_unknown_part(self):
        self.assertRaises(snap_modules.UnknownPartError, snap_modules.is_supported_combination, part="RF6000",
                          core="2.7.2")

    def test_is_supported_combination_for_unknown_core(self):
        self.assertRaises(snap_modules.UnknownCoreError, snap_modules.is_supported_combination, part="RF100",
                          core="2.4.99")
